from django.core.exceptions import PermissionDenied

from graphene_django.types import DjangoObjectType

from aleksis.core.schema.base import (
    BaseBatchCreateMutation,
    BaseBatchDeleteMutation,
    BaseBatchPatchMutation,
    DjangoFilterMixin,
    PermissionsTypeMixin,
)

from ..models import GradeChoice, GradeSet


class GradeSetType(PermissionsTypeMixin, DjangoFilterMixin, DjangoObjectType):
    class Meta:
        model = GradeSet
        fields = ("id", "name", "grade_choices")
        filter_fields = {
            "id": ["exact"],
            "name": ["icontains", "exact"],
        }

    @classmethod
    def get_queryset(cls, queryset, info):
        if info.context.user.has_perm("maka.fetch_gradesets_rule"):
            return queryset
        raise PermissionDenied()


class GradeSetBatchCreateMutation(BaseBatchCreateMutation):
    class Meta:
        model = GradeSet
        permissions = ("maka.create_gradeset_rule",)
        only_fields = ("name",)


class GradeSetBatchDeleteMutation(BaseBatchDeleteMutation):
    class Meta:
        model = GradeSet
        permissions = ("maka.delete_gradeset_rule",)


class GradeSetBatchPatchMutation(BaseBatchPatchMutation):
    class Meta:
        model = GradeSet
        permissions = ("maka.edit_gradeset_rule",)
        only_fields = (
            "id",
            "name",
        )


class GradeChoiceType(PermissionsTypeMixin, DjangoFilterMixin, DjangoObjectType):
    class Meta:
        model = GradeChoice
        fields = ("id", "grade_set", "name", "value", "order")
        filter_fields = {
            "id": ["exact"],
            "name": ["icontains", "exact"],
            "value": ["icontains", "exact"],
        }

    @classmethod
    def get_queryset(cls, queryset, info):
        if info.context.user.has_perm("maka.fetch_gradechoices_rule"):
            return queryset.order_by("order")
        raise PermissionDenied()


class GradeChoiceBatchCreateMutation(BaseBatchCreateMutation):
    class Meta:
        model = GradeChoice
        permissions = ("maka.create_gradechoice_rule",)
        only_fields = (
            "grade_set",
            "name",
            "value",
            "order",
        )


class GradeChoiceBatchDeleteMutation(BaseBatchDeleteMutation):
    class Meta:
        model = GradeChoice
        permissions = ("maka.delete_gradechoice_rule",)


class GradeChoiceBatchPatchMutation(BaseBatchPatchMutation):
    class Meta:
        model = GradeChoice
        permissions = ("maka.edit_gradechoice_rule",)
        only_fields = (
            "id",
            "grade_set",
            "name",
            "value",
            "order",
        )
